/********************************************************************
//        NAME- ErrorCodeController
// DESCRIPTION- controls all the functions of the error code panel
//       NOTES- Inherits from DataRetriever
********************************************************************/

/********************************************************************
// ErrorCodeController() : constructor
// Instantiates the ErrorCodeController object
********************************************************************/
function ErrorLogController()
{
  this.saveCommand   = 'commands/saveErrorLog.php';
  this.viewURL = 'views/errorLog.php';

  this.clearButton = getElement('clearErrorLog');
  this.clearAdminButton = getElement('clearAdminErrorLog');
  this.saveButton = getElement('saveErrorLog');

  this.viewController = new ViewController('errorLogFrame');
  this.clearRequest = new ServerRequest('commands/clearErrorLog.php');
  this.clearAllRequest = new ServerRequest('commands/clearAllLogs.php');
  
  this.shadowBox = this.viewController.shadowBox;
  
  this.init();
}

ErrorLogController.prototype.init = function init()
{
  this.clearButton.controller = this;
  this.clearButton.action = function ()
  {
    Frontend.clearLogDialog.promptToClearLog(translate('MSG_CLEAR'),translate('MSG_CLEAR_ERROR_LOGS'),this.controller);
  }
  this.clearButton.disable();

  if (CONFIG.ADMIN_MODE) {
    this.clearAdminButton.controller = this;
    this.clearAdminButton.action = function ()
    {
      Frontend.clearLogDialog.promptToClearLog(translate('MSG_CLEAR'),translate('MSG_CLEAR_ERROR_LOGS'),this.controller);
    }
  }

  this.saveButton.controller = this;
  this.saveButton.action = function ()
  {
    Frontend.saveLogDialog.promptToSaveLog(translate('MSG_SAVE_LOGS'),translate('MSG_SAVE_ERROR_LOGS'),this.controller);
  }
  this.saveButton.disable();
  
  this.clearRequest.controller = this;
  this.clearRequest.callback = function ()
  {
    this.controller.refreshFrame();
  }
  
  this.clearAllRequest.controller = this;
  this.clearAllRequest.callback = function ()
  {
    this.controller.refreshFrame();
  }
}

ErrorLogController.prototype.refreshFrame = function refreshFrame()
{
  this.saveButton.disable();
  this.clearButton.disable();
  this.viewController.setURL(this.viewURL);
}

ErrorLogController.prototype.clear = function clear()
{
  this.clearRequest.send();
}

ErrorLogController.prototype.clearAll = function clearAll()
{
  this.clearAllRequest.send();
}

ErrorLogController.prototype.appendSaveInfo = function appendSaveInfo(request)
{
  return false;
}

ErrorLogController.prototype.save = function save()
{
  var header = (translate('MSG_SAVE') + ' ' + translate('MSG_ERROR_LOG'));
  if (CONFIG.IS_OFFLINE)
  {
    Frontend.saveDialog.openDialog(header, 'errorlog', this);
  }
  else
  {
    var query = 
    {
      attachment : 1,
      filename   : 'errorlog'
    }
    Frontend.clientSave(this.saveCommand, query, header);
  }
}

ErrorLogController.prototype.saveAll = function saveAll() {
  var header = translate('MSG_SAVE_ALL_LOGS');
  if (CONFIG.IS_OFFLINE) {
    Frontend.saveDialog.openDialog(header, 'alllogs', this);
  }
  else {
    var query =
    {
      attachment: 1,
      filename: 'alllogs'
    }
    Frontend.clientSave('commands/saveAllLogs.php', query, header);
  }
}
